@@ -18,9 +18,9 @@ from argparse import ArgumentParser, _ArgumentGroup
 from pipes import quote
 from typing import Callable, Dict, List, Optional, TypeVar, Union
 
-from toil.batchSystems.abstractGridEngineBatchSystem import (
-    AbstractGridEngineBatchSystem,
-)
+from toil.batchSystems.abstractGridEngineBatchSystem import \
+    AbstractGridEngineBatchSystem
+from toil.batchSystems.options import OptionSetter
 from toil.lib.misc import CalledProcessErrorStderr, call_command
 
 logger = logging.getLogger(__name__)
@@ -355,6 +355,6 @@ class SlurmBatchSystem(AbstractGridEngineBatchSystem):
 
     OptionType = TypeVar('OptionType')
     @classmethod
-    def setOptions(cls, setOption: Callable[[str, Optional[Callable[[str], OptionType]], Optional[Callable[[OptionType], None]], Optional[OptionType]], None]) -> None:
+    def setOptions(cls, setOption: OptionSetter) -> None:
         setOption("allocate_mem", bool, default=False)
 
