@@ -24,8 +24,9 @@ import tempfile
 import time
 import uuid
 from contextlib import contextmanager
-from urllib.parse import quote, unquote, ParseResult
 from typing import IO, Iterator, List, Optional, Union, overload
+from urllib.parse import ParseResult, quote, unquote
+
 if sys.version_info >= (3, 8):
     from typing import Literal
 else:
@@ -33,14 +34,15 @@ else:
 
 from toil.fileStores import FileID
 from toil.job import TemporaryID
-from toil.jobStores.abstractJobStore import (
-    AbstractJobStore,
-    JobStoreExistsException,
-    NoSuchFileException,
-    NoSuchJobException,
-    NoSuchJobStoreException,
-)
-from toil.lib.io import AtomicFileCreate, atomic_copy, atomic_copyobj, robust_rmtree
+from toil.jobStores.abstractJobStore import (AbstractJobStore,
+                                             JobStoreExistsException,
+                                             NoSuchFileException,
+                                             NoSuchJobException,
+                                             NoSuchJobStoreException)
+from toil.lib.io import (AtomicFileCreate,
+                         atomic_copy,
+                         atomic_copyobj,
+                         robust_rmtree)
 
 logger = logging.getLogger(__name__)
 
@@ -490,6 +492,7 @@ class FileJobStore(AbstractJobStore):
                 # It worked!
                 return
             except OSError as e:
+                # For the list of the possible errno codes, see: https://linux.die.net/man/2/symlink
                 if e.errno == errno.EEXIST:
                     # Overwrite existing file, emulating shutil.copyfile().
                     os.unlink(local_path)
@@ -498,7 +501,12 @@ class FileJobStore(AbstractJobStore):
                     os.symlink(jobStoreFilePath, local_path)
                     # Now we succeeded and don't need to copy
                     return
+                elif e.errno == errno.EPERM:
+                    # On some filesystems, the creation of symbolic links is not possible.
+                    # In this case, we try to make a hard link.
+                    pass
                 else:
+                    logger.error(f"Unexpected OSError when reading file '{jobStoreFilePath}' from job store")
                     raise
 
         # If we get here, symlinking isn't an option.
@@ -516,6 +524,7 @@ class FileJobStore(AbstractJobStore):
                 # It worked!
                 return
             except OSError as e:
+                # For the list of the possible errno codes, see: https://linux.die.net/man/2/link
                 if e.errno == errno.EEXIST:
                     # Overwrite existing file, emulating shutil.copyfile().
                     os.unlink(local_path)
@@ -528,10 +537,20 @@ class FileJobStore(AbstractJobStore):
                     # It's a cross-device link even though it didn't appear to be.
                     # Just keep going and hit the file copy case.
                     pass
+                elif e.errno == errno.EPERM:
+                    # On some filesystems, hardlinking could be disallowed by permissions.
+                    # In this case, we also fall back to making a complete copy.
+                    pass
+                elif e.errno == errno.ELOOP:
+                    # Too many symbolic links were encountered. Just keep going and hit the
+                    # file copy case.
+                    pass
+                elif e.errno == errno.EMLINK:
+                    # The maximum number of links to file is reached. Just keep going and
+                    # hit the file copy case.
+                    pass
                 else:
-                    logger.critical('Unexpected OSError when reading file from job store')
-                    logger.critical('jobStoreFilePath: ' + jobStoreFilePath + ' ' + str(os.path.exists(jobStoreFilePath)))
-                    logger.critical('localFilePath: ' + local_path + ' ' + str(os.path.exists(local_path)))
+                    logger.error(f"Unexpected OSError when reading file '{jobStoreFilePath}' from job store")
                     raise
 
         # If we get here, neither a symlink nor a hardlink will work.
