@@ -15,10 +15,10 @@ import configparser
 import json
 import logging
 import os.path
+import platform
 import subprocess
 import tempfile
 import textwrap
-import platform
 from abc import ABC, abstractmethod
 from functools import total_ordering
 from typing import Any, Dict, List, Optional, Set, Tuple, Union
@@ -513,7 +513,7 @@ class AbstractProvisioner(ABC):
             # Holds strings like "ssh-rsa actualKeyData" for keys to authorize (independently of cloud provider's system)
             self.sshPublicKeys = []
 
-        def addFile(self, path: str, filesystem: str = 'root', mode: Union[str, int] = '0755', contents: str = ''):
+        def addFile(self, path: str, filesystem: str = 'root', mode: Union[str, int] = '0755', contents: str = '', append: bool = False):
             """
             Make a file on the instance with the given filesystem, mode, and contents.
 
@@ -527,7 +527,12 @@ class AbstractProvisioner(ABC):
 
             contents = 'data:,' + quote(contents.encode('utf-8'))
 
-            self.files.append({'path': path, 'filesystem': filesystem, 'mode': mode, 'contents': {'source': contents}})
+            ignition_file = {'path': path, 'filesystem': filesystem, 'mode': mode, 'contents': {'source': contents}}
+
+            if append:
+                ignition_file["append"] = append
+
+            self.files.append(ignition_file)
 
         def addUnit(self, name: str, enabled: bool = True, contents: str = ''):
             """
@@ -606,12 +611,22 @@ class AbstractProvisioner(ABC):
         """
         Add a service to prepare and mount local scratch volumes.
         """
+
+        # TODO: when
+        # https://www.flatcar.org/docs/latest/setup/storage/mounting-storage/
+        # describes how to collect all the ephemeral disks declaratively and
+        # make Ignition RAID them, stop doing it manually. Might depend on real
+        # solution for https://github.com/coreos/ignition/issues/1126
+        #
+        # TODO: check what kind of instance this is, and what ephemeral volumes
+        # *should* be there, and declaratively RAID and mount them.
         config.addFile("/home/core/volumes.sh", contents=textwrap.dedent("""\
             #!/bin/bash
             set -x
             ephemeral_count=0
             drives=()
-            directories=(toil mesos docker kubelet cwl)
+            # Directories are relative to /var
+            directories=(lib/toil lib/mesos lib/docker lib/kubelet lib/cwl tmp)
             for drive in /dev/xvd{a..z} /dev/nvme{0..26}n1; do
                 echo "checking for ${drive}"
                 if [ -b $drive ]; then
@@ -643,7 +658,7 @@ class AbstractProvisioner(ABC):
             if (("$ephemeral_count" == "0" )); then
                 echo "no ephemeral drive"
                 for directory in "${directories[@]}"; do
-                    sudo mkdir -p /var/lib/$directory
+                    sudo mkdir -p /var/$directory
                 done
                 exit 0
             fi
@@ -664,11 +679,12 @@ class AbstractProvisioner(ABC):
                 sudo mount /dev/md0 /mnt/ephemeral
             fi
             for directory in "${directories[@]}"; do
-                sudo mkdir -p /mnt/ephemeral/var/lib/$directory
-                sudo mkdir -p /var/lib/$directory
-                sudo mount --bind /mnt/ephemeral/var/lib/$directory /var/lib/$directory
+                sudo mkdir -p /mnt/ephemeral/var/$directory
+                sudo mkdir -p /var/$directory
+                sudo mount --bind /mnt/ephemeral/var/$directory /var/$directory
             done
             """))
+        # TODO: Make this retry?
         config.addUnit("volume-mounting.service", contents=textwrap.dedent("""\
             [Unit]
             Description=mounts ephemeral volumes & bind mounts toil directories
@@ -713,7 +729,10 @@ class AbstractProvisioner(ABC):
             WantedBy=multi-user.target
             '''))
 
-    def addToilService(self, config: InstanceConfiguration, role: str, keyPath: str = None, preemptable: bool = False):
+    def toil_service_env_options(self) -> str:
+        return "-e TMPDIR=/var/tmp"
+
+    def add_toil_service(self, config: InstanceConfiguration, role: str, keyPath: str = None, preemptable: bool = False):
         """
         Add the Toil leader or worker service to an instance configuration.
 
@@ -766,6 +785,9 @@ class AbstractProvisioner(ABC):
             entryPointArgs = " ".join(["'" + customDockerInitCommand + "'", entryPoint, entryPointArgs])
             entryPoint = "customDockerInit.sh"
 
+        # Set up the service. Make sure to make it default to using the
+        # actually-big temp directory of /var/tmp (see
+        # https://systemd.io/TEMPORARY_DIRECTORIES/).
         config.addUnit(f"toil-{role}.service", contents=textwrap.dedent(f'''\
             [Unit]
             Description=toil-{role} container
@@ -778,15 +800,17 @@ class AbstractProvisioner(ABC):
             ExecStartPre=-/usr/bin/docker rm toil_{role}
             ExecStartPre=-/usr/bin/bash -c '{customInitCmd()}'
             ExecStart=/usr/bin/docker run \\
+                {self.toil_service_env_options()} \\
                 --entrypoint={entryPoint} \\
                 --net=host \\
                 --init \\
                 -v /var/run/docker.sock:/var/run/docker.sock \\
-                -v /var/run/user:/var/run/user \\
+                -v /run/lock:/run/lock \\
                 -v /var/lib/mesos:/var/lib/mesos \\
                 -v /var/lib/docker:/var/lib/docker \\
                 -v /var/lib/toil:/var/lib/toil \\
                 -v /var/lib/cwl:/var/lib/cwl \\
+                -v /var/tmp:/var/tmp \\
                 -v /tmp:/tmp \\
                 -v /opt:/opt \\
                 -v /etc/kubernetes:/etc/kubernetes \\
@@ -809,6 +833,7 @@ class AbstractProvisioner(ABC):
             CRICTL_VERSION="v1.17.0",
             CNI_DIR="/opt/cni/bin",
             DOWNLOAD_DIR="/opt/bin",
+            SETUP_STATE_DIR="/etc/toil/kubernetes",
             # This is the version of Kubernetes to use
             # Get current from: curl -sSL https://dl.k8s.io/release/stable.txt
             # Make sure it is compatible with the kubelet.service unit we ship, or update that too.
@@ -878,9 +903,18 @@ class AbstractProvisioner(ABC):
             ''').format(**values))
 
         # Before we let the kubelet try to start, we have to actually download it (and kubeadm)
+        # We set up this service so it can restart on failure despite not
+        # leaving a process running, see
+        # <https://github.com/openshift/installer/pull/604> and
+        # <https://github.com/litew/droid-config-ham/commit/26601d85d9d972dc1560096db1c419fd6fd9b238>
+        # We use a forking service with RemainAfterExit, since that lets
+        # restarts work if the script fails. We also use a condition which
+        # treats the service as successful and skips it if it made a file to
+        # say it already ran.
         config.addFile("/home/core/install-kubernetes.sh", contents=textwrap.dedent('''\
             #!/usr/bin/env bash
             set -e
+            FLAG_FILE="{SETUP_STATE_DIR}/install-kubernetes.done"
 
             # Make sure we have Docker enabled; Kubeadm later might complain it isn't.
             systemctl enable docker.service
@@ -893,6 +927,9 @@ class AbstractProvisioner(ABC):
             cd {DOWNLOAD_DIR}
             curl -L --remote-name-all https://storage.googleapis.com/kubernetes-release/release/{KUBERNETES_VERSION}/bin/linux/{ARCHITECTURE}/{{kubeadm,kubelet,kubectl}}
             chmod +x {{kubeadm,kubelet,kubectl}}
+
+            mkdir -p "{SETUP_STATE_DIR}"
+            touch "$FLAG_FILE"
             ''').format(**values))
         config.addUnit("install-kubernetes.service", contents=textwrap.dedent('''\
             [Unit]
@@ -900,15 +937,19 @@ class AbstractProvisioner(ABC):
             Wants=network-online.target
             After=network-online.target
             Before=kubelet.service
+            ConditionPathExists=!{SETUP_STATE_DIR}/install-kubernetes.done
 
             [Service]
-            Type=oneshot
-            Restart=no
             ExecStart=/usr/bin/bash /home/core/install-kubernetes.sh
+            Type=forking
+            RemainAfterExit=yes
+            Restart=on-failure
+            RestartSec=5s
 
             [Install]
             WantedBy=multi-user.target
-            '''))
+            RequiredBy=kubelet.service
+            ''').format(**values))
 
         # Now we should have the kubeadm command, and the bootlooping kubelet
         # waiting for kubeadm to configure it.
@@ -1008,6 +1049,8 @@ class AbstractProvisioner(ABC):
             #!/usr/bin/env bash
             set -e
 
+            FLAG_FILE="{SETUP_STATE_DIR}/create-kubernetes-cluster.done"
+
             export PATH="$PATH:{DOWNLOAD_DIR}"
 
             # We need the kubelet being restarted constantly by systemd while kubeadm is setting up.
@@ -1017,7 +1060,10 @@ class AbstractProvisioner(ABC):
             # We also need to set the hostname for 'kubeadm init' to work properly.
             /bin/sh -c "/usr/bin/hostnamectl set-hostname $(curl -s http://169.254.169.254/latest/meta-data/hostname)"
 
-            kubeadm init --config /home/core/kubernetes-leader.yml
+            if [[ ! -e /etc/kubernetes/admin.conf ]] ; then
+                # Only run this once, it isn't idempotent
+                kubeadm init --config /home/core/kubernetes-leader.yml
+            fi
 
             mkdir -p $HOME/.kube
             cp /etc/kubernetes/admin.conf $HOME/.kube/config
@@ -1043,6 +1089,9 @@ class AbstractProvisioner(ABC):
             echo "JOIN_TOKEN=$(kubeadm token create --ttl 0)" >>/etc/kubernetes/worker.ini
             echo "JOIN_CERT_HASH=sha256:$(openssl x509 -pubkey -in /etc/kubernetes/pki/ca.crt | openssl rsa -pubin -outform der 2>/dev/null | openssl dgst -sha256 -hex | sed 's/^.* //')" >>/etc/kubernetes/worker.ini
             echo "JOIN_ENDPOINT=$(hostname):6443" >>/etc/kubernetes/worker.ini
+
+            mkdir -p "{SETUP_STATE_DIR}"
+            touch "$FLAG_FILE"
             ''').format(**values))
         config.addUnit("create-kubernetes-cluster.service", contents=textwrap.dedent('''\
             [Unit]
@@ -1051,15 +1100,19 @@ class AbstractProvisioner(ABC):
             After=docker.service
             Before=toil-leader.service
             # Can't be before kubelet.service because Kubelet has to come up as we run this.
+            ConditionPathExists=!{SETUP_STATE_DIR}/create-kubernetes-cluster.done
 
             [Service]
-            Type=oneshot
-            Restart=no
             ExecStart=/usr/bin/bash /home/core/create-kubernetes-cluster.sh
+            Type=forking
+            RemainAfterExit=yes
+            Restart=on-failure
+            RestartSec=5s
 
             [Install]
             WantedBy=multi-user.target
-            '''))
+            RequiredBy=toil-leader.service
+            ''').format(**values))
 
         # We also need a node cleaner service
         config.addFile("/home/core/cleanup-nodes.sh", contents=textwrap.dedent('''\
@@ -1085,7 +1138,8 @@ class AbstractProvisioner(ABC):
         config.addUnit("cleanup-nodes.service", contents=textwrap.dedent('''\
             [Unit]
             Description=Remove scaled-in nodes
-            After=install-kubernetes.service
+            After=create-kubernetes-cluster.service
+            Requires=create-kubernetes-cluster.service
             [Service]
             ExecStart=/home/core/cleanup-nodes.sh
             Restart=always
@@ -1142,6 +1196,7 @@ class AbstractProvisioner(ABC):
         config.addFile("/home/core/join-kubernetes-cluster.sh", contents=textwrap.dedent('''\
             #!/usr/bin/env bash
             set -e
+            FLAG_FILE="{SETUP_STATE_DIR}/join-kubernetes-cluster.done"
 
             export PATH="$PATH:{DOWNLOAD_DIR}"
 
@@ -1150,6 +1205,9 @@ class AbstractProvisioner(ABC):
             systemctl start kubelet
 
             kubeadm join {JOIN_ENDPOINT} --config /home/core/kubernetes-worker.yml
+
+            mkdir -p "{SETUP_STATE_DIR}"
+            touch "$FLAG_FILE"
             ''').format(**values))
 
         config.addUnit("join-kubernetes-cluster.service", contents=textwrap.dedent('''\
@@ -1158,15 +1216,19 @@ class AbstractProvisioner(ABC):
             After=install-kubernetes.service
             After=docker.service
             # Can't be before kubelet.service because Kubelet has to come up as we run this.
+            Requires=install-kubernetes.service
+            ConditionPathExists=!{SETUP_STATE_DIR}/join-kubernetes-cluster.done
 
             [Service]
-            Type=oneshot
-            Restart=no
             ExecStart=/usr/bin/bash /home/core/join-kubernetes-cluster.sh
+            Type=forking
+            RemainAfterExit=yes
+            Restart=on-failure
+            RestartSec=5s
 
             [Install]
             WantedBy=multi-user.target
-            '''))
+            ''').format(**values))
 
     def _getIgnitionUserData(self, role, keyPath=None, preemptable=False, architecture='amd64'):
         """
@@ -1195,7 +1257,7 @@ class AbstractProvisioner(ABC):
 
         if self.clusterType == 'mesos' or role == 'leader':
             # Leaders, and all nodes in a Mesos cluster, need a Toil service
-            self.addToilService(config, role, keyPath, preemptable)
+            self.add_toil_service(config, role, keyPath, preemptable)
 
         if role == 'worker' and self._leaderWorkerAuthentication is not None:
             # We need to connect the worker to the leader.
