@@ -21,14 +21,23 @@ import textwrap
 import time
 import uuid
 from functools import wraps
-from typing import Any, Callable, Collection, Dict, Iterable, List, Optional, Set
+from shlex import quote
+from typing import (Any,
+                    Callable,
+                    Collection,
+                    Dict,
+                    Iterable,
+                    List,
+                    Optional,
+                    Set)
 from urllib.parse import unquote
 
 # We need these to exist as attributes we can get off of the boto object
 import boto.ec2
 import boto.iam
 import boto.vpc
-from boto.ec2.blockdevicemapping import BlockDeviceMapping as Boto2BlockDeviceMapping
+from boto.ec2.blockdevicemapping import \
+    BlockDeviceMapping as Boto2BlockDeviceMapping
 from boto.ec2.blockdevicemapping import BlockDeviceType as Boto2BlockDeviceType
 from boto.ec2.instance import Instance as Boto2Instance
 from boto.exception import BotoServerError, EC2ResponseError
@@ -37,8 +46,11 @@ from botocore.exceptions import ClientError
 
 from toil.lib.aws import zone_to_region
 from toil.lib.aws.ami import get_flatcar_ami
-from toil.lib.aws.utils import create_s3_bucket
+from toil.lib.aws.iam import (CLUSTER_LAUNCHING_PERMISSIONS,
+                              get_policy_permissions,
+                              policy_permissions_allow)
 from toil.lib.aws.session import AWSConnectionManager
+from toil.lib.aws.utils import create_s3_bucket
 from toil.lib.conversions import human2bytes
 from toil.lib.ec2 import (a_short_time,
                           create_auto_scaling_group,
@@ -53,20 +65,17 @@ from toil.lib.ec2nodes import InstanceType
 from toil.lib.generatedEC2Lists import E2Instances
 from toil.lib.memoize import memoize
 from toil.lib.misc import truncExpBackoff
-from toil.lib.retry import (
-    ErrorCondition,
-    get_error_body,
-    get_error_code,
-    get_error_status,
-    old_retry,
-    retry,
-)
-from toil.provisioners import NoSuchClusterException, ClusterCombinationNotSupportedException
-from toil.provisioners.abstractProvisioner import (
-    AbstractProvisioner,
-    ManagedNodesNotSupportedException,
-    Shape,
-)
+from toil.lib.retry import (ErrorCondition,
+                            get_error_body,
+                            get_error_code,
+                            get_error_status,
+                            old_retry,
+                            retry)
+from toil.provisioners import (ClusterCombinationNotSupportedException,
+                               NoSuchClusterException)
+from toil.provisioners.abstractProvisioner import (AbstractProvisioner,
+                                                   ManagedNodesNotSupportedException,
+                                                   Shape)
 from toil.provisioners.aws import get_best_aws_zone
 from toil.provisioners.node import Node
 
@@ -300,6 +309,11 @@ class AWSProvisioner(AbstractProvisioner):
         :return: None
         """
 
+        # First, pre-flight-check our permissions before making anything.
+        if not policy_permissions_allow(get_policy_permissions(region=self._region), CLUSTER_LAUNCHING_PERMISSIONS):
+            # Function prints a more specific warning to the log, but give some context.
+            logger.warning('Toil may not be able to properly launch (or destroy!) your cluster.')
+
         leader_type = E2Instances[leaderNodeType]
 
         if self.clusterType == 'kubernetes':
@@ -324,25 +338,27 @@ class AWSProvisioner(AbstractProvisioner):
             self._leader_subnet = self._get_default_subnet(self._zone)
 
         profileArn = awsEc2ProfileArn or self._createProfileArn()
+
         # the security group name is used as the cluster identifier
         createdSGs = self._createSecurityGroups()
         bdms = self._getBoto3BlockDeviceMappings(leader_type, rootVolSize=leaderStorage)
 
-        userData = self._getIgnitionUserData('leader', architecture=self._architecture)
-
         # Make up the tags
         self._tags = {'Name': self.clusterName,
                       'Owner': owner,
                       _TAG_KEY_TOIL_CLUSTER_NAME: self.clusterName}
 
+        if userTags is not None:
+            self._tags.update(userTags)
+
+        #All user specified tags have been set
+        userData = self._getIgnitionUserData('leader', architecture=self._architecture)
+
         if self.clusterType == 'kubernetes':
             # All nodes need a tag putting them in the cluster.
             # This tag needs to be on there before the a leader can finish its startup.
             self._tags['kubernetes.io/cluster/' + self.clusterName] = ''
 
-        if userTags is not None:
-            self._tags.update(userTags)
-
         # Make tags for the leader specifically
         leader_tags = dict(self._tags)
         leader_tags[_TAG_KEY_TOIL_NODE_TYPE] = 'leader'
@@ -373,7 +389,11 @@ class AWSProvisioner(AbstractProvisioner):
         leader.reload()
 
         if leader.public_ip_address is None:
-            raise RuntimeError("AWS did not assign a public IP to the cluster leader! Leader is lost!")
+            # Sometimes AWS just fails to assign a public IP when we really need one.
+            # But sometimes people intend to use private IPs only in Toil-managed clusters.
+            # TODO: detect if we have a route to the private IP and fail fast if not.
+            logger.warning("AWS did not assign a public IP to the cluster leader. If you aren't "
+                           "connected to the private subnet, cluster setup will fail!")
 
         # Remember enough about the leader to let us launch workers in its
         # cluster.
@@ -392,6 +412,14 @@ class AWSProvisioner(AbstractProvisioner):
         # Download credentials
         self._setLeaderWorkerAuthentication(leaderNode)
 
+    def toil_service_env_options(self) -> str:
+        """
+        Set AWS tags in user docker container
+        """
+        config = super().toil_service_env_options()
+        instance_base_tags = json.dumps(self._tags)
+        return config + " -e TOIL_AWS_TAGS=" + quote(instance_base_tags)
+
     def _get_worker_subnets(self) -> List[str]:
         """
         Get all worker subnets we should balance across, as a flat list.
@@ -595,7 +623,7 @@ class AWSProvisioner(AbstractProvisioner):
         # Do the workers after the ASGs because some may belong to ASGs
         logger.info('Terminating any remaining workers ...')
         removed = False
-        instances = self._getNodesInCluster(both=True)
+        instances = self._get_nodes_in_cluster(include_stopped_nodes=True)
         spotIDs = self._getSpotRequestIDs()
         if spotIDs:
             self.aws.boto2(self._region, 'ec2').cancel_spot_instance_requests(request_ids=spotIDs)
@@ -845,7 +873,7 @@ class AWSProvisioner(AbstractProvisioner):
 
     def getProvisionedWorkers(self, instance_type: Optional[str] = None, preemptable: Optional[bool] = None) -> List[Node]:
         assert self._leaderPrivateIP
-        entireCluster = self._getNodesInCluster(instance_type=instance_type, both=True)
+        entireCluster = self._get_nodes_in_cluster(instance_type=instance_type)
         logger.debug('All nodes in cluster: %s', entireCluster)
         workerInstances = [i for i in entireCluster if i.private_ip_address != self._leaderPrivateIP]
         logger.debug('All workers found in cluster: %s', workerInstances)
@@ -901,7 +929,7 @@ class AWSProvisioner(AbstractProvisioner):
         """
         Get the Boto 2 instance for the cluster's leader.
         """
-        instances = self._getNodesInCluster(both=True)
+        instances = self._get_nodes_in_cluster(include_stopped_nodes=True)
         instances.sort(key=lambda x: x.launch_time)
         try:
             leader = instances[0]  # assume leader was launched first
@@ -964,7 +992,7 @@ class AWSProvisioner(AbstractProvisioner):
         self._terminateIDs(instanceIDs)
         logger.info('... Waiting for instance(s) to shut down...')
         for instance in instances:
-            wait_transition(instance, {'pending', 'running', 'shutting-down'}, 'terminated')
+            wait_transition(instance, {'pending', 'running', 'shutting-down', 'stopping', 'stopped'}, 'terminated')
         logger.info('Instance(s) terminated.')
 
     @awsRetry
@@ -1065,24 +1093,34 @@ class AWSProvisioner(AbstractProvisioner):
         return bdms
 
     @awsRetry
-    def _getNodesInCluster(self, instance_type: Optional[str] = None, preemptable=False, both=False) -> List[Boto2Instance]:
+    def _get_nodes_in_cluster(self, instance_type: Optional[str] = None, include_stopped_nodes=False) -> List[Boto2Instance]:
         """
         Get Boto2 instance objects for all nodes in the cluster.
         """
 
-        allInstances = self.aws.boto2(self._region, 'ec2').get_only_instances(filters={'instance.group-name': self.clusterName})
+        all_instances = self.aws.boto2(self._region, 'ec2').get_only_instances(filters={'instance.group-name': self.clusterName})
+
         def instanceFilter(i):
             # filter by type only if nodeType is true
             rightType = not instance_type or i.instance_type == instance_type
             rightState = i.state == 'running' or i.state == 'pending'
+            if include_stopped_nodes:
+                rightState = rightState or i.state == 'stopping' or i.state == 'stopped'
             return rightType and rightState
-        filteredInstances = [i for i in allInstances if instanceFilter(i)]
-        if not preemptable and not both:
-            return [i for i in filteredInstances if i.spot_instance_request_id is None]
-        elif preemptable and not both:
-            return [i for i in filteredInstances if i.spot_instance_request_id is not None]
-        elif both:
-            return filteredInstances
+
+        return [i for i in all_instances if instanceFilter(i)]
+
+    def _filter_nodes_in_cluster(self, instance_type: Optional[str] = None, preemptable: bool = False) -> List[Boto2Instance]:
+        """
+        Get Boto2 instance objects for the nodes in the cluster filtered by preemptability.
+        """
+
+        instances = self._get_nodes_in_cluster(instance_type, include_stopped_nodes=False)
+
+        if preemptable:
+            return [i for i in instances if i.spot_instance_request_id is not None]
+
+        return [i for i in instances if i.spot_instance_request_id is None]
 
     def _getSpotRequestIDs(self) -> List[str]:
         """
@@ -1710,6 +1748,4 @@ class AWSProvisioner(AbstractProvisioner):
                 iam.add_role_to_instance_profile(profile.instance_profile_name, iamRoleName)
                 logger.debug("Associated role %s with profile", iamRoleName)
 
-
         return profile_arn
-
