@@ -1,11 +1,11 @@
 # Modified from: https://github.com/common-workflow-language/workflow-service
 import functools
 import json
-import os
 import logging
+import os
 import tempfile
 from abc import abstractmethod
-from typing import Optional, List, Dict, Any, Tuple, Callable
+from typing import Any, Callable, Dict, List, Optional, Tuple, Union
 from urllib.parse import urldefrag
 
 import connexion  # type: ignore
@@ -13,6 +13,10 @@ from werkzeug.utils import secure_filename
 
 logger = logging.getLogger(__name__)
 
+# Define a type for WES task log entries in responses
+# TODO: make this a typed dict with all the WES task log field names and their types.
+TaskLog = Dict[str, Union[str, int, None]]
+
 
 class VersionNotImplementedException(Exception):
     """
@@ -256,7 +260,8 @@ class WESBackend:
         else:
             raise MalformedRequestException("Missing 'workflow_url' in submission")
 
-        if "workflow_params" not in body:
-            raise MalformedRequestException("Missing 'workflow_params' in submission")
+        if "workflow_params" in body and not isinstance(body["workflow_params"], dict):
+            # They sent us something silly like "workflow_params": "5"
+            raise MalformedRequestException("Got a 'workflow_params' which does not decode to a JSON object")
 
         return temp_dir, body
