@@ -31,34 +31,34 @@ from contextlib import contextmanager
 from inspect import getsource
 from shutil import which
 from textwrap import dedent
-from typing import (
-    Any,
-    Callable,
-    Dict,
-    Generator,
-    List,
-    Optional,
-    Tuple,
-    Type,
-    TypeVar,
-    Union,
-    cast,
-)
+from typing import (Any,
+                    Callable,
+                    Dict,
+                    Generator,
+                    List,
+                    Optional,
+                    Tuple,
+                    Type,
+                    TypeVar,
+                    Union,
+                    cast)
 from unittest.util import strclass
 from urllib.error import HTTPError, URLError
 from urllib.request import urlopen
 
 import pytz
+
 if sys.version_info >= (3, 8):
     from typing import Literal
 else:
     from typing_extensions import Literal
 
 from toil import ApplianceImageNotFound, applianceSelf, toilPackageDirPath
+from toil.lib.accelerators import have_working_nvidia_smi, have_working_nvidia_docker_runtime
+from toil.lib.aws import running_on_ec2
 from toil.lib.iterables import concat
 from toil.lib.memoize import memoize
 from toil.lib.threading import ExceptionalThread, cpu_count
-from toil.lib.aws import running_on_ec2
 from toil.version import distVersion
 
 logger = logging.getLogger(__name__)
@@ -570,7 +570,28 @@ def needs_docker(test_item: MT) -> MT:
         return test_item
     else:
         return unittest.skip("Install docker to include this test.")(test_item)
+        
+def needs_local_cuda(test_item: MT) -> MT:
+    """
+    Use as a decorator before test classes or methods to only run them if
+    a CUDA setup legible to cwltool (i.e. providing userspace nvidia_smi) is present.
+    """
+    test_item = _mark_test('local_cuda', test_item)
+    if have_working_nvidia_smi():
+        return test_item
+    else:
+        return unittest.skip("Install nvidia-smi, an nvidia proprietary driver, and a CUDA-capable nvidia GPU to include this test.")(test_item)
 
+def needs_docker_cuda(test_item: MT) -> MT:
+    """
+    Use as a decorator before test classes or methods to only run them if
+    a CUDA setup is available through Docker.
+    """
+    test_item = _mark_test('docker_cuda', test_item)
+    if have_working_nvidia_docker_runtime():
+        return test_item
+    else:
+        return unittest.skip("Install nvidia-container-runtime on your Docker server and configure an 'nvidia' runtime to include this test.")(test_item)
 
 def needs_encryption(test_item: MT) -> MT:
     """
