@@ -7,9 +7,10 @@ def binaryStringFn(job, depth, message=""):
         job.addChildJobFn(binaryStringFn, depth-1, message + "0")
         job.addChildJobFn(binaryStringFn, depth-1, message + "1")
     else:
-        job.log("Binary string: {}".format(message))
+        job.log(f"Binary string: {message}")
 
-if __name__=="__main__":
+
+if __name__ == "__main__":
     options = Job.Runner.getDefaultOptions("./toilWorkflowRun")
     options.logLevel = "INFO"
     options.clean = "always"
