@@ -11,6 +11,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+import logging
 import os
 import signal
 import time
@@ -24,6 +25,8 @@ from toil.leader import FailedJobsException
 from toil.lib.threading import cpu_count
 from toil.test import ToilTest, slow
 
+logger = logging.getLogger(__name__)
+
 
 class DeferredFunctionTest(ToilTest, metaclass=ABCMeta):
     """Test the deferred function system."""
@@ -166,10 +169,12 @@ class DeferredFunctionTest(ToilTest, metaclass=ABCMeta):
 
     def testBatchSystemCleanupCanHandleWorkerDeaths(self):
         """
-        Create a non-local files. Create a job that registers a deferred job to delete the file
-        and then kills itself.
+        Create some non-local files. Create a job that registers a deferred
+        function to delete the file and then kills its worker.
 
-        Assert that the file is missing after the pipeline fails.
+        Assert that the file is missing after the pipeline fails, because we're
+        using a single-machine batch system and the leader's batch system
+        cleanup will find and run the deferred function.
         """
 
         # There can be no retries
@@ -184,6 +189,9 @@ class DeferredFunctionTest(ToilTest, metaclass=ABCMeta):
         open(nonLocalFile2, 'w').close()
         assert os.path.exists(nonLocalFile1)
         assert os.path.exists(nonLocalFile2)
+        # We only use the "A" job here, and we fill in the first file, so all
+        # it will do is defer deleting the second file, delete the first file,
+        # and die.
         A = Job.wrapJobFn(_testNewJobsCanHandleOtherJobDeaths_A,
                           files=(nonLocalFile1, nonLocalFile2))
         try:
@@ -254,9 +262,13 @@ def _deleteFile(nonLocalFile, nlf=None):
     :param str nlf:
     :return: None
     """
+    logger.debug("Removing file: %s", nonLocalFile)
     os.remove(nonLocalFile)
+    logger.debug("Successfully removed file: %s", nonLocalFile)
     if nlf is not None:
+        logger.debug("Removing file: %s", nlf)
         os.remove(nlf)
+        logger.debug("Successfully removed file: %s", nlf)
 
 def _testNewJobsCanHandleOtherJobDeaths_A(job, files):
     """
@@ -266,11 +278,13 @@ def _testNewJobsCanHandleOtherJobDeaths_A(job, files):
     :param tuple files: the tuple of the two files to work with
     :return: None
     """
+
     # Write the pid to files[1] such that we can be sure that this process has died before
     # we spawn the next job that will do the cleanup.
     with open(files[1], 'w') as fileHandle:
         fileHandle.write(str(os.getpid()))
     job.defer(_deleteFile, files[1])
+    logger.info("Deferred delete of %s", files[1])
     while os.stat(files[0]).st_size == 0:
         time.sleep(0.5)
     os.remove(files[0])
