@@ -15,14 +15,14 @@
 from toil.common import Config
 from toil.job import CheckpointJobDescription, JobDescription
 from toil.jobStores.fileJobStore import FileJobStore
-from toil.test import ToilTest, travis_test
+from toil.test import ToilTest
 from toil.worker import nextChainable
 
 
 class WorkerTests(ToilTest):
     """Test miscellaneous units of the worker."""
     def setUp(self):
-        super(WorkerTests, self).setUp()
+        super().setUp()
         path = self._getTestJobStorePath()
         self.jobStore = FileJobStore(path)
         self.config = Config()
@@ -30,7 +30,6 @@ class WorkerTests(ToilTest):
         self.jobStore.initialize(self.config)
         self.jobNumber = 0
 
-    @travis_test
     def testNextChainable(self):
         """Make sure chainable/non-chainable jobs are identified correctly."""
         def createTestJobDesc(memory, cores, disk, preemptable, checkpoint):
@@ -45,10 +44,10 @@ class WorkerTests(ToilTest):
             jobDesc = descClass(requirements={'memory': memory, 'cores': cores, 'disk': disk, 'preemptable': preemptable}, jobName=name)
 
             # Assign an ID
-            self.jobStore.assignID(jobDesc)
+            self.jobStore.assign_job_id(jobDesc)
 
             # Save and return the JobDescription
-            return self.jobStore.create(jobDesc)
+            return self.jobStore.create_job(jobDesc)
 
         for successorType in ['addChild', 'addFollowOn']:
             # Try with the branch point at both child and follow-on stages
