@@ -9,18 +9,16 @@ from urllib.request import urlretrieve
 from toil.test import ToilTest, needs_docker, needs_java, slow
 from toil.version import exactPython
 from toil.wdl.utils import get_analyzer
-from toil.wdl.wdl_functions import (
-    basename,
-    glob,
-    parse_cores,
-    parse_disk,
-    parse_memory,
-    process_infile,
-    read_csv,
-    read_tsv,
-    select_first,
-    size,
-)
+from toil.wdl.wdl_functions import (basename,
+                                    glob,
+                                    parse_cores,
+                                    parse_disk,
+                                    parse_memory,
+                                    process_infile,
+                                    read_csv,
+                                    read_tsv,
+                                    select_first,
+                                    size)
 
 
 class ToilWdlIntegrationTest(ToilTest):
